#!/usr/bin/isympy
# -*- coding: utf-8 -*-

from mechanics import *

#Design length            3.000 m  Midship location                      1.500 m
#Length over all          2.880 m  Relative water density                1.025
#Design beam              0.700 m  Mean shell thickness                 0.0000 m
#Beam over all            0.640 m  Appendage coefficient                1.0000
#Design draft             0.150 m
#Draft │ Lwl  │ Bwl  │ Volume │ Displ. │ LCB  │  Cb  │  Cb  │  Am  │  Cm  │  Aw  │  Cw  │ LCF  │  Cp  │  S   │
#  m   │  m   │  m   │   m3   │ tonnes │  m   │      │      │ m^2  │      │ m^2  │      │  m   │      │ m^2  │
# 0.150│ 2.611│ 0.602│   0.099│   0.101│ 1.501│ 0.419│ 0.419│ 0.074│ 0.822│ 0.967│ 0.616│ 1.500│ 0.510│ 1.271│

#Loa,Boa,Dd,Lwl,H,Hk,Th,Fb,a,D,Cb,v,A,b,r,fbmax=symbols("Loa,Boa,Dd,Lwl,H,Hk,Th,Fb,a,D,Cb,v,A,b,r,fbmax",real=True,positive=True)
Loa=2.88 * m
Boa=0.64 * m
Dd=0.15 * m
Lwl=2.611 * m

#The depth H in [m] is the vertical distance between
#the canoe body bottom and the top edge of the deck,
#measured at the side of the craft halfway along LWL.
#this is the highest point in midship.
#Without catamaran superstructures!
#The ship is lowest at midship BTW.
# it would better be 0.33
H=0.25 * m

#depth of keel
Hk=0.2 * m

#The hull draught TH is the maximum draught of the
#canoe body of the yacht.
# The draft from hydrostatic report before Lwl and Bwl drop to 0

Th = 0.31 * m

#In the case of open or partially-decked craft, the free-
#board is the minimum distance between the flotation
#plane and the upper edge of the gunwale or an opening
#in the hull without a watertight closure. For decked
#craft the freeboard is to be measured to the upper edge
#of the deck at its lowest point.

Fb=(0.25-Dd)*m

#The frame spacing a will be measured from moulding
#edge to moulding edge of frames.

#I do not have frames, man!
#maybe the distance of the two bulkheads
#a=1.8 * m
a=0.3 * m

#The displacement weight D in [t] is the weight of the
#craft in the ready to operate condition, corresponding
#to the sum of the lightweight and the deadweight.
D=0.101 * k* kg

#Block coefficient CB
#Moulded block coefficient at full displacement
#draught T, based on the rule length L:
# I have it from hydrostatic data

Cb=0.419

#The speed v is the expected maximum speed in knots
#[kn] of the craft in the ready to operate condition in
#smooth water.
v=20 * kn
v=7 * kn


#total surface area of rudder without that of skeg, in [m2]
A=0.2 * m * m
#mean height of rudder surface
b=0.2 * m
#rudder radius
r=0.2 * m

#max stress of material used
fbmax=30*N/mm/mm

##### end of ship config ####

#material constant
#k = 12 as standard value for teak, kambala, oak,
#         sipo-mahogany
#  = 16 as standard value for less firm wood, e. g.
#         khaya-mahogany, sound pine

K=20

#The length LH in [m] of the hull is the horizontal dis-
#tance between the foremost and the aftermost part of
#the craft. The length includes structural and integral
#parts of the craft.

Lh=Loa

#The scantling length is determined as follows:
#             L H + L WL
#      L =  --------------- [m]
#                   2

L= (Lh + Lwl)/2

#The beam B in [m] is the maximum breadth of the
#craft measured from one shell outer edge to the other,
#disregarding any rubbing strakes etc.

B=Boa
#The depth H1 in [m] is the depth H increased by 1/6 of
#the depth Hk of the keel, measured at the side of the
#craft halfway along LWL, as in Figs. 1.1 and 1.2.


H1=H+Hk/6

#The draught T in [m] is the vertical distance, meas-
#ured halfway along LWL, between the flotation plane
#of the craft in the ready to operate condition and the
#bottom edge of the keel.
# Okay, it should be the design draft.
T=Dd

#Hull loadings
#we take the maximum of motorcraft and sailing and motorsailers, and for fore and aft
Pdbmm={
'motorfore': 2.7* L *k*N/m**3 + 3.29 *k*N/m**2, 
'sailfore': 3.29*L  *k*N/m**3- 1.41 *k*N/m**2,
'motoraft': 2.16*L  *k*N/m**3+ 2.63 *k*N/m**2,
'sailaft': 2.63 *L  *k*N/m**3- 1.13 *k*N/m**2,
}
Pdbm=max(Pdbmm.values())

#shell side design loading [kN/m^2]
# motor fore, sailing fore, motor aft, sail aft
Pdsm=max(1.88*L*k*N/m**3 + 1.76*k*N/m**2, 2.06*L*k*N/m**3 - 2.94*k*N/m**2, 1.5*L*k*N/m**3 + 1.41*k*N/m**2, 1.65*L*k*N/m**3 - 2.35*k*N/m**2)

#correction factors for speed:

#speed factor. all correction factors include this.
spf = ((v/(Lwl**(0.5)))**0.5)*(kn**-0.5)*(m**0.25)
#shell bottom:
Fvb = max(1.0,0.34*spf+0.355)
#shell side:
Fvs = max(1.0,(0.024*(spf**2)+0.91)*(1.018-0.0024*L/m))
#internal structural members, floors
Fvf = max(1.0,(0.78 * spf -0.48)*(1.335-0.001*L/m))
#     Web frame at CL, Bottom longitudinal frames FVL 
Fvbw = Fvl = max(1.0,0.075 * (spf ** 2) + 0.73 )
#Transverse frames Webs at side
Fvsf = Fvsw = max(1.0,(0.1* (spf ** 2) + 0.52) * (1.19 - 0.01 * L/m))
# Side longitudinal frames
Fvsl = max(1.0,(0.14* (spf ** 2) + 0.47) * (1.07 - 0.008 * L/m))
#maximum correction factor for speed
Fv=max(Fvb,Fvs,Fvbw,Fvsf,Fvsl)

#max designable speed
Vmax = 12 * (L ** (1.0/4))*kn*m**-0.25

#Deck and superstructure loadings kN/m^2
#           Main deck
#                        deck 1
#  Cabins   h  0,5 m
#                         wall
#                       deck 1, 2
#Deckhouses h > 0,5 m side wall 2
#                     front wall
#We just get the maximum of all, and pretend h'=0, so we get the maximum load

Pdd = max(0.26*L/m + 8.24, 0.235*L/m + 7.42, 0.26*L/m + 8.24,0.235*L/m + 7.42,0.26*L/m + 8.24, 1.25* (0.26*L/m + 8.24),4.0)*k*N/m**2

if (spf ** 2 >= 10.8) or (( (spf ** 2) > 3.6) and (D/kg/k> 0.094*(L**2/m**2 - 15.8))):
    print """Warning! design speed is out of bounds!"""

# rudder force
# we choose K1,Ch, and K2 as maximum
K1 = 1.53
Ch = 132
K2 = 1.2
# rudder force [N]
Cr = K1 * K2 * Ch * v * v * A /kn**2/m**2 *N

# rudder torsional moment [Nm]
Qr = Cr * r


#scantling number
s=L*(B/3+H1)

#absolute maximum design load. Maximum correction factor for speed * maximum of design loads [kN/m**2]
Pd = Fv * max(Pdbm,Pdsm,Pdd)

#Now compute shell thickness. 
#First come up with some beam approximation.
# A beam supported on both ends, length of frame spacing, and bearing the design load.
# supports:
R1,R2,x=symbols("R1 R2 x")

P1=Pd
# less conservative: shell bottom for midship * shell bottom speed coeff
#P1=Pdbmm['sailaft']*Fvb
l=a
Fp1=integrate(P1,(x,0,x))
eq1=integrate(Fp1,(x,0,l))-R1*l
R1=solve(eq1,R1)[0]
R2=R1 #symmetric settings

shear=beamcomponents([R1,0,-P1*x,l,R2],x)

moment=integrate(shear,(x,-oo,x))

#shearbare=shear.subs(measures)
#momentbare=moment.subs(measures)
#mplot2d([shearbare,momentbare],[x,-1,3,1000])

#zeropoint=solve(shear,x)[0]
zeropoint=l/2
maxmoment=moment.subs(x,zeropoint)

c=symbols("c ")
eq2 = maxmoment*c/((2*c)**3)*12 - fbmax
solve(eq2,c)

#shell thickness
# it is roughly proportional to frame space, square root of boat length, 4th root of boat speed, and inverse square root of fbmax, btw
h=2*max(map(abs,solve(eq2,c)))

